/// <reference types="node" />
import { EventEmitter } from 'events';
import { Context, KeyboardBuilder, MessageContext } from 'vk-io';
import { IAutoGeneratePagesOptions, IPagesBuilderOptions, IResetListenTimeoutOptions, ISetDefaultButtonsOptions, ITrigger, Page, PagesContext, PageSendMethod, PageSendMethodUnion, PagesStorage } from './types';
export declare const pagesStorage: PagesStorage;
export declare class PagesBuilder<C extends Context = Context> extends EventEmitter {
    #private;
    constructor({ context }: IPagesBuilderOptions<C>);
    /**
     * Method for initial pages setup
     */
    setPages(pages: Page | Page[]): this;
    /**
     * Method for adding pages to end of collection
     */
    addPages(pages: Page | Page[]): this;
    /**
     * Method for auto generate pages
     */
    autoGeneratePages({ items, countPerPage }: IAutoGeneratePagesOptions): this;
    /**
     * Method for open page
     */
    setPage(pageNumber?: number): Promise<MessageContext>;
    /**
     * Method for auto reset timeout after page change
     */
    autoResetTimeout(status?: boolean): this;
    /**
     * Method for set pages pagination format
     */
    setPagesPaginationFormat(format?: string): this;
    /**
     * Method for set infinite pages switching
     */
    setInfinityLoop(status?: boolean): this;
    /**
     * Method for set pages header
     */
    setPagesHeader(header?: string): this;
    /**
     * Method for set pages footer
     */
    setPagesFooter(footer?: string): this;
    /**
     * Method for set send pages method
     */
    setSendMethod(method?: PageSendMethod | PageSendMethodUnion): this;
    private getPage;
    /**
     * Method for set updates listen time
     */
    setListenTime(time?: number): this;
    /**
     * Method for reset current updates listen timeout
     */
    resetListenTimeout({ isFirstBuild }?: IResetListenTimeoutOptions): void;
    /**
     * Method for set updates listen users
     */
    setListenUsers(users?: number | number[]): this;
    /**
     * Method for add updates listen users
     */
    addListenUsers(users?: number | number[]): this;
    /**
     * Method for stop listen updates
     */
    stopListen(): Promise<void>;
    /**
     * Method for set default buttons
     */
    setDefaultButtons({ buttons, type }?: ISetDefaultButtonsOptions): this;
    /**
     * Method for update keyboard
     */
    updateKeyboard(keyboard?: KeyboardBuilder): this;
    /**
     * Method for initial triggers setup
     */
    setTriggers(triggers: ITrigger | ITrigger[]): this;
    /**
     * Method for add triggers
     */
    addTriggers(triggers: ITrigger | ITrigger[]): this;
    /**
     * @private
     */
    executeTrigger(trigger: ITrigger['name']): ReturnType<ITrigger['callback']>;
    /**
     * Method for build & send pages
     */
    build(loader?: MessageContext | PagesContext<MessageContext>): Promise<this>;
    /**
     * Context getter
     */
    get context(): PagesContext<C>;
    /**
     * Id getter
     */
    get id(): string;
    /**
     * Pages collection getter
     */
    get pages(): Page[];
    /**
     * Header getter
     */
    get header(): string;
    /**
     * Footer getter
     */
    get footer(): string;
    /**
     * Current page getter
     */
    get currentPage(): number;
    /**
     * Infinity loop status getter
     */
    get infinityLoop(): boolean;
    /**
     * Auto reset timeout status getter
     */
    get resetTimeout(): boolean;
    /**
     * Pages pagination format getter
     */
    get pagesPaginationFormat(): string;
    /**
     * Send method getter
     */
    get sendMethod(): "editMessage" | "send" | PageSendMethod;
    /**
     * Listen time getter
     */
    get listenTime(): number;
    /**
     * Listen users getter
     */
    get listenUsers(): number[];
    /**
     * Sent message context getter
     */
    get sent(): PagesContext<MessageContext<import("vk-io").ContextDefaultState>> | undefined;
    set sent(context: PagesContext<MessageContext<import("vk-io").ContextDefaultState>> | undefined);
    /**
     * Keyboard getter
     */
    get keyboard(): KeyboardBuilder;
    /**
     * @private
     */
    saveContext(): void;
    /**
     * Message handle middleware
     *
     * @private
     */
    messageMiddleware(context: PagesContext<C>): Promise<void>;
}
