"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pagesStorage = exports.PagesBuilder = void 0;

var _events = require("events");

var _vkIo = require("vk-io");

var _utils = require("./utils");

var _types = require("./types");

function _classPrivateMethodInitSpec(obj, privateSet) { _checkPrivateRedeclaration(obj, privateSet); privateSet.add(obj); }

function _classPrivateFieldInitSpec(obj, privateMap, value) { _checkPrivateRedeclaration(obj, privateMap); privateMap.set(obj, value); }

function _checkPrivateRedeclaration(obj, privateCollection) { if (privateCollection.has(obj)) { throw new TypeError("Cannot initialize the same private elements twice on an object"); } }

function _classPrivateMethodGet(receiver, privateSet, fn) { if (!privateSet.has(receiver)) { throw new TypeError("attempted to get private field on non-instance"); } return fn; }

function _classPrivateFieldGet(receiver, privateMap) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "get"); return _classApplyDescriptorGet(receiver, descriptor); }

function _classApplyDescriptorGet(receiver, descriptor) { if (descriptor.get) { return descriptor.get.call(receiver); } return descriptor.value; }

function _classPrivateFieldSet(receiver, privateMap, value) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "set"); _classApplyDescriptorSet(receiver, descriptor, value); return value; }

function _classExtractFieldDescriptor(receiver, privateMap, action) { if (!privateMap.has(receiver)) { throw new TypeError("attempted to " + action + " private field on non-instance"); } return privateMap.get(receiver); }

function _classApplyDescriptorSet(receiver, descriptor, value) { if (descriptor.set) { descriptor.set.call(receiver, value); } else { if (!descriptor.writable) { throw new TypeError("attempted to set read only private field"); } descriptor.value = value; } }

const pagesStorage = new Map();
exports.pagesStorage = pagesStorage;

var _id = /*#__PURE__*/new WeakMap();

var _context = /*#__PURE__*/new WeakMap();

var _pages = /*#__PURE__*/new WeakMap();

var _header = /*#__PURE__*/new WeakMap();

var _footer = /*#__PURE__*/new WeakMap();

var _currentPage = /*#__PURE__*/new WeakMap();

var _infinityLoop = /*#__PURE__*/new WeakMap();

var _resetTimeout = /*#__PURE__*/new WeakMap();

var _pagesPaginationFormat = /*#__PURE__*/new WeakMap();

var _sendMethod = /*#__PURE__*/new WeakMap();

var _listenTime = /*#__PURE__*/new WeakMap();

var _listenTimeout = /*#__PURE__*/new WeakMap();

var _listenUsers = /*#__PURE__*/new WeakMap();

var _triggers = /*#__PURE__*/new WeakMap();

var _sent = /*#__PURE__*/new WeakMap();

var _keyboard = /*#__PURE__*/new WeakMap();

var _cleanUpKeyboard = /*#__PURE__*/new WeakSet();

var _executeAction = /*#__PURE__*/new WeakSet();

class PagesBuilder extends _events.EventEmitter {
  /**
   * Builder ID
   *
   * @private
   */

  /**
   * Received message context
   *
   * @private
   */

  /**
   * Pages collection
   *
   * @private
   */

  /**
   * Pages header
   *
   * @private
   */

  /**
   * Pages footer
   *
   * @private
   */

  /**
   * Current page
   *
   * @private
   */

  /**
   * Infinite pages switching
   *
   * @private
   */

  /**
   * Auto reset timeout after page change
   *
   * @private
   */

  /**
   * Pagination format for pages
   *
   * @private
   */

  /**
   * Method for send messages
   *
   * @private
   */

  /**
   * Updates listen time
   *
   * @private
   */

  /**
   * Listen timeout context
   *
   * @private
   */

  /**
   * Updates listen users
   *
   * @private
   */

  /**
   * Triggers collection
   *
   * @private
   */

  /**
   * Sent message context
   *
   * @private
   */

  /**
   * Builder keyboard
   *
   * @private
   */
  constructor({
    context
  }) {
    super();

    _classPrivateMethodInitSpec(this, _executeAction);

    _classPrivateMethodInitSpec(this, _cleanUpKeyboard);

    _classPrivateFieldInitSpec(this, _id, {
      writable: true,
      value: (0, _utils.randomString)(6)
    });

    _classPrivateFieldInitSpec(this, _context, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _pages, {
      writable: true,
      value: []
    });

    _classPrivateFieldInitSpec(this, _header, {
      writable: true,
      value: ''
    });

    _classPrivateFieldInitSpec(this, _footer, {
      writable: true,
      value: ''
    });

    _classPrivateFieldInitSpec(this, _currentPage, {
      writable: true,
      value: 1
    });

    _classPrivateFieldInitSpec(this, _infinityLoop, {
      writable: true,
      value: true
    });

    _classPrivateFieldInitSpec(this, _resetTimeout, {
      writable: true,
      value: false
    });

    _classPrivateFieldInitSpec(this, _pagesPaginationFormat, {
      writable: true,
      value: '%c / %m'
    });

    _classPrivateFieldInitSpec(this, _sendMethod, {
      writable: true,
      value: _types.PageSendMethod.SEND
    });

    _classPrivateFieldInitSpec(this, _listenTime, {
      writable: true,
      value: 5 * 60 * 1000
    });

    _classPrivateFieldInitSpec(this, _listenTimeout, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _listenUsers, {
      writable: true,
      value: []
    });

    _classPrivateFieldInitSpec(this, _triggers, {
      writable: true,
      value: new Map()
    });

    _classPrivateFieldInitSpec(this, _sent, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _keyboard, {
      writable: true,
      value: _vkIo.Keyboard.builder().inline()
    });

    _classPrivateFieldSet(this, _context, context);

    this.setDefaultButtons();
  }
  /**
   * Method for initial pages setup
   */


  setPages(pages) {
    _classPrivateFieldSet(this, _pages, !Array.isArray(pages) ? [pages] : pages);

    return this;
  }
  /**
   * Method for adding pages to end of collection
   */


  addPages(pages) {
    _classPrivateFieldSet(this, _pages, _classPrivateFieldGet(this, _pages).concat(pages));

    return this;
  }
  /**
   * Method for auto generate pages
   */


  autoGeneratePages({
    items,
    countPerPage = 10
  }) {
    const chunks = (0, _utils.chunk)(items, countPerPage);
    this.setPages(chunks.map(chunk => ({
      message: chunk.join('\n')
    })));
    return this;
  }
  /**
   * Method for open page
   */


  async setPage(pageNumber = _classPrivateFieldGet(this, _currentPage)) {
    _classPrivateFieldSet(this, _currentPage, pageNumber);

    if (_classPrivateFieldGet(this, _resetTimeout)) {
      this.resetListenTimeout();
    }

    this.saveContext();
    const page = await this.getPage(pageNumber);
    this.emit('page_set', pageNumber);
    return (0, _utils.editMessage)(this, page);
  }
  /**
   * Method for auto reset timeout after page change
   */


  autoResetTimeout(status = true) {
    _classPrivateFieldSet(this, _resetTimeout, status);

    this.saveContext();
    return this;
  }
  /**
   * Method for set pages pagination format
   */


  setPagesPaginationFormat(format = '%c / %m') {
    _classPrivateFieldSet(this, _pagesPaginationFormat, format);

    this.saveContext();
    return this;
  }
  /**
   * Method for set infinite pages switching
   */


  setInfinityLoop(status = true) {
    _classPrivateFieldSet(this, _infinityLoop, status);

    this.saveContext();
    return this;
  }
  /**
   * Method for set pages header
   */


  setPagesHeader(header = '') {
    _classPrivateFieldSet(this, _header, header);

    this.saveContext();
    return this;
  }
  /**
   * Method for set pages footer
   */


  setPagesFooter(footer = '') {
    _classPrivateFieldSet(this, _footer, footer);

    this.saveContext();
    return this;
  }
  /**
   * Method for set send pages method
   */


  setSendMethod(method = _types.PageSendMethod.SEND) {
    _classPrivateFieldSet(this, _sendMethod, method);

    this.saveContext();
    return this;
  }

  async getPage(pageNumber = _classPrivateFieldGet(this, _currentPage)) {
    var _page$keyboard;

    let page = _classPrivateFieldGet(this, _pages)[pageNumber - 1];

    if (typeof page === 'function') {
      page = await page(_classPrivateFieldGet(this, _context));
    }

    if (typeof page === 'string') {
      page = {
        message: page
      };
    }

    let keyboard = (_page$keyboard = page.keyboard) !== null && _page$keyboard !== void 0 ? _page$keyboard : _classPrivateFieldGet(this, _keyboard);

    if (typeof keyboard === 'function') {
      keyboard = await keyboard(_classPrivateFieldGet(this, _context));
    }

    if (!_classPrivateFieldGet(this, _infinityLoop)) {
      keyboard = _classPrivateMethodGet(this, _cleanUpKeyboard, _cleanUpKeyboard2).call(this, keyboard);
    } else {
      keyboard = JSON.parse(keyboard.toString());
    }

    const pagePagination = _classPrivateFieldGet(this, _pagesPaginationFormat).replace('%c', String(pageNumber)).replace('%m', String(_classPrivateFieldGet(this, _pages).length));

    const hasKeyboardButtons = keyboard.buttons.length;
    return { ...page,
      message: `${_classPrivateFieldGet(this, _header)}${page.message}${_classPrivateFieldGet(this, _footer)}\n\n${pagePagination}`,
      keyboard: hasKeyboardButtons ? JSON.stringify(keyboard) : undefined
    };
  }
  /**
   * Method for set updates listen time
   */


  setListenTime(time = 5 * 60 * 1000) {
    _classPrivateFieldSet(this, _listenTime, time);

    this.saveContext();
    return this;
  }
  /**
   * Method for reset current updates listen timeout
   */


  resetListenTimeout({
    isFirstBuild = false
  } = {}) {
    if (_classPrivateFieldGet(this, _listenTimeout) || isFirstBuild) {
      clearTimeout(_classPrivateFieldGet(this, _listenTimeout));

      _classPrivateFieldSet(this, _listenTimeout, setTimeout(this.stopListen.bind(this), _classPrivateFieldGet(this, _listenTime)));

      this.saveContext();
      this.emit('listen_reset_timeout');
    }
  }
  /**
   * Method for set updates listen users
   */


  setListenUsers(users = []) {
    if (!Array.isArray(users)) {
      users = [users];
    }

    _classPrivateFieldSet(this, _listenUsers, users);

    this.saveContext();
    return this;
  }
  /**
   * Method for add updates listen users
   */


  addListenUsers(users = []) {
    _classPrivateFieldSet(this, _listenUsers, _classPrivateFieldGet(this, _listenUsers).concat(users));

    this.saveContext();
    return this;
  }
  /**
   * Method for stop listen updates
   */


  async stopListen() {
    if (_classPrivateFieldGet(this, _listenTimeout)) {
      clearTimeout(_classPrivateFieldGet(this, _listenTimeout));
      pagesStorage.delete(_classPrivateFieldGet(this, _id));
      this.emit('listen_stop', _classPrivateFieldGet(this, _currentPage));
      const page = await this.getPage(_classPrivateFieldGet(this, _currentPage));
      (0, _utils.editMessage)(this, { ...page,
        keyboard: JSON.stringify({})
      }, true);
    }
  }
  /**
   * Method for set default buttons
   */


  setDefaultButtons({
    buttons = [_types.DefaultButtonAction.FIRST, _types.DefaultButtonAction.BACK, _types.DefaultButtonAction.STOP, _types.DefaultButtonAction.NEXT, _types.DefaultButtonAction.LAST],
    type = _types.DefaultButtonsType.TEXT
  } = {}) {
    const keyboard = _vkIo.Keyboard.builder().inline(true);

    const defaultButtons = new Map([[_types.DefaultButtonAction.FIRST, '⏪'], [_types.DefaultButtonAction.BACK, '◀️'], [_types.DefaultButtonAction.STOP, '⏹'], [_types.DefaultButtonAction.NEXT, '▶️'], [_types.DefaultButtonAction.LAST, '⏩']]);
    buttons.forEach(button => {
      const label = defaultButtons.get(button);

      if (label) {
        const buttonObject = {
          label,
          payload: {
            builder_action: button,
            builder_id: _classPrivateFieldGet(this, _id)
          }
        };
        keyboard[`${type}Button`](buttonObject);
      }
    });
    keyboard.row();

    _classPrivateFieldSet(this, _keyboard, keyboard);

    this.saveContext();
    return this;
  }
  /**
   * Method for update keyboard
   */


  updateKeyboard(keyboard) {
    _classPrivateFieldSet(this, _keyboard, keyboard !== null && keyboard !== void 0 ? keyboard : _vkIo.Keyboard.builder().inline(true));

    this.saveContext();
    return this;
  }
  /**
   * Method for initial triggers setup
   */


  setTriggers(triggers) {
    if (!Array.isArray(triggers)) {
      triggers = [triggers];
    }

    _classPrivateFieldSet(this, _triggers, new Map(triggers.map(({
      name,
      callback
    }) => [name, callback])));

    this.saveContext();
    return this;
  }
  /**
   * Method for add triggers
   */


  addTriggers(triggers) {
    if (!Array.isArray(triggers)) {
      triggers = [triggers];
    }

    triggers.forEach(({
      name,
      callback
    }) => {
      _classPrivateFieldGet(this, _triggers).set(name, callback);
    });
    this.saveContext();
    return this;
  }
  /**
   * @private
   */


  executeTrigger(trigger) {
    const triggerAction = _classPrivateFieldGet(this, _triggers).get(trigger);

    if (triggerAction) {
      this.emit('trigger_execute', trigger);
      return triggerAction(_classPrivateFieldGet(this, _context));
    }

    return Promise.resolve();
  }
  /**
   * Method for build & send pages
   */


  build(loader) {
    return new Promise(async (resolve, reject) => {
      if (_classPrivateFieldGet(this, _pages).length === 0) {
        return reject(new Error('Pages not set'));
      }

      if (loader) {
        _classPrivateFieldSet(this, _sent, loader);

        if (_classPrivateFieldGet(this, _sendMethod) === _types.PageSendMethod.SEND) {
          loader.deleteMessage({
            delete_for_all: 1
          }).catch(() => null);
        }
      }

      const page = await this.getPage(1);
      (loader ? (0, _utils.editMessage)(this, page) : _classPrivateFieldGet(this, _context).send(page)).then(context => {
        if (typeof context === 'object') {
          _classPrivateFieldSet(this, _sent, context);

          this.saveContext();
        }

        this.emit('listen_start');
        this.resetListenTimeout({
          isFirstBuild: true
        });
        resolve(this);
      }).catch(reject);
    });
  }
  /**
   * Context getter
   */


  get context() {
    return _classPrivateFieldGet(this, _context);
  }
  /**
   * Id getter
   */


  get id() {
    return _classPrivateFieldGet(this, _id);
  }
  /**
   * Pages collection getter
   */


  get pages() {
    return _classPrivateFieldGet(this, _pages);
  }
  /**
   * Header getter
   */


  get header() {
    return _classPrivateFieldGet(this, _header);
  }
  /**
   * Footer getter
   */


  get footer() {
    return _classPrivateFieldGet(this, _footer);
  }
  /**
   * Current page getter
   */


  get currentPage() {
    return _classPrivateFieldGet(this, _currentPage);
  }
  /**
   * Infinity loop status getter
   */


  get infinityLoop() {
    return _classPrivateFieldGet(this, _infinityLoop);
  }
  /**
   * Auto reset timeout status getter
   */


  get resetTimeout() {
    return _classPrivateFieldGet(this, _resetTimeout);
  }
  /**
   * Pages pagination format getter
   */


  get pagesPaginationFormat() {
    return _classPrivateFieldGet(this, _pagesPaginationFormat);
  }
  /**
   * Send method getter
   */


  get sendMethod() {
    return _classPrivateFieldGet(this, _sendMethod);
  }
  /**
   * Listen time getter
   */


  get listenTime() {
    return _classPrivateFieldGet(this, _listenTime);
  }
  /**
   * Listen users getter
   */


  get listenUsers() {
    return _classPrivateFieldGet(this, _listenUsers);
  }
  /**
   * Sent message context getter
   */


  get sent() {
    return _classPrivateFieldGet(this, _sent);
  }

  set sent(context) {
    _classPrivateFieldSet(this, _sent, context);
  }
  /**
   * Keyboard getter
   */


  get keyboard() {
    return _classPrivateFieldGet(this, _keyboard).clone();
  }
  /**
   * @private
   */


  saveContext() {
    pagesStorage.set(_classPrivateFieldGet(this, _id), this);
  }
  /**
   * @private
   */


  /**
   * Message handle middleware
   *
   * @private
   */
  async messageMiddleware(context) {
    _classPrivateFieldSet(this, _context, context);

    const payload = (context === null || context === void 0 ? void 0 : context.messagePayload) || (context === null || context === void 0 ? void 0 : context.eventPayload);
    const action = payload === null || payload === void 0 ? void 0 : payload.builder_action;
    const builderId = payload === null || payload === void 0 ? void 0 : payload.builder_id;
    const page = Number(payload === null || payload === void 0 ? void 0 : payload.builder_page);
    const trigger = payload === null || payload === void 0 ? void 0 : payload.builder_trigger;

    if ((action || page || trigger) && builderId === _classPrivateFieldGet(this, _id)) {
      if (_classPrivateFieldGet(this, _listenUsers).length && !_classPrivateFieldGet(this, _listenUsers).includes(context.senderId)) {
        return;
      }

      const isMessageEvent = context.type === 'message_event';

      if (!isMessageEvent) {
        (0, _utils.markAsRead)(context);
      }

      if (action) {
        _classPrivateMethodGet(this, _executeAction, _executeAction2).call(this, action);
      } else if (page) {
        if (page >= 1 && page <= _classPrivateFieldGet(this, _pages).length) {
          if (page === _classPrivateFieldGet(this, _currentPage) && !isMessageEvent) {
            return;
          }

          this.setPage(page);
        }
      }

      if (trigger) {
        await this.executeTrigger(trigger);
        this.setPage();
      }
    }
  }

}

exports.PagesBuilder = PagesBuilder;

function _cleanUpKeyboard2(keyboardBuilder) {
  const keyboard = JSON.parse(keyboardBuilder.toString());
  keyboard.buttons = keyboard.buttons.map(row => row.filter(button => {
    try {
      const payload = JSON.parse(button.action.payload);
      const payloadKeys = Object.keys(payload);
      const isCustomButton = payloadKeys.findIndex(key => key !== 'builder_action' && key !== 'builder_id') !== -1;

      if (isCustomButton) {
        return true;
      }

      const {
        builder_action
      } = payload;

      if (builder_action) {
        const isFirstPage = _classPrivateFieldGet(this, _currentPage) === 1;

        const isLastPage = _classPrivateFieldGet(this, _currentPage) === _classPrivateFieldGet(this, _pages).length;

        const hasMoreThanTwoPages = _classPrivateFieldGet(this, _pages).length > 2;
        const hasOnePage = _classPrivateFieldGet(this, _pages).length === 1;
        return !hasOnePage ? !(isFirstPage && (builder_action === _types.DefaultButtonAction.FIRST || builder_action === _types.DefaultButtonAction.BACK || !hasMoreThanTwoPages && builder_action === _types.DefaultButtonAction.LAST) || isLastPage && (builder_action === _types.DefaultButtonAction.LAST || builder_action === _types.DefaultButtonAction.NEXT || !hasMoreThanTwoPages && builder_action === _types.DefaultButtonAction.FIRST)) : false;
      } // eslint-disable-next-line no-empty

    } catch {}

    return true;
  })).filter(row => row.length);
  return keyboard;
}

function _executeAction2(action) {
  this.emit('page_action_execute', action);
  const isFirstPage = _classPrivateFieldGet(this, _currentPage) === 1;

  const isLastPage = _classPrivateFieldGet(this, _currentPage) === _classPrivateFieldGet(this, _pages).length;

  switch (action) {
    case _types.DefaultButtonAction.FIRST:
      if (isFirstPage) {
        if (_classPrivateFieldGet(this, _infinityLoop)) {
          this.setPage(_classPrivateFieldGet(this, _pages).length);
        }

        return;
      }

      this.setPage(1);
      break;

    case _types.DefaultButtonAction.BACK:
      if (isFirstPage) {
        if (_classPrivateFieldGet(this, _infinityLoop)) {
          this.setPage(_classPrivateFieldGet(this, _pages).length);
        }

        return;
      }

      this.setPage(_classPrivateFieldGet(this, _currentPage) - 1);
      break;

    case _types.DefaultButtonAction.STOP:
      this.stopListen();
      break;

    case _types.DefaultButtonAction.NEXT:
      if (isLastPage) {
        if (_classPrivateFieldGet(this, _infinityLoop)) {
          this.setPage(1);
        }

        return;
      }

      this.setPage(_classPrivateFieldGet(this, _currentPage) + 1);
      break;

    case _types.DefaultButtonAction.LAST:
      if (isLastPage) {
        if (_classPrivateFieldGet(this, _infinityLoop)) {
          this.setPage(1);
        }

        return;
      }

      this.setPage(_classPrivateFieldGet(this, _pages).length);
      break;
  }
}